@@ -12,12 +12,12 @@ En2 <- function (D, traps, mask, detectpar, noccasions, detectfn =
         stop ("En2 is for detector types multi, proximity and count only")
     if (is.character(detectfn))
         detectfn <- match.arg(detectfn)
-    detectfn <- secr:::valid.detectfn(detectfn, valid = c(0,1,2,14:19))
+    detectfn <- secr:::secr_valid.detectfn(detectfn, valid = c(0,1,2,14:19))
     dfc <- dfcast (detectfn, detectpar)  # transforms detectfn 0 to 14, 2 to 16
     detectfn <- dfc$detectfn
     detectpar <- dfc$detectpar
-    detectpars <- unlist(detectpar[secr:::parnames(detectfn)])
-    dettype <- secr:::detectorcode(traps, noccasions = noccasions)[1]
+    detectpars <- unlist(detectpar[secr:::secr_parnames(detectfn)])
+    dettype <- secr:::secr_detectorcode(traps, noccasions = noccasions)[1]
     D <- rep(D, length.out = nrow(mask)) * attr(mask, 'area')  # per cell
     temp <- En2cpp (
         as.integer(dettype),
